#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include "timer.h"
#include "max.h"
#include "message.h"
#include "balai.h"

#define LOG_CONDITION debug_clock

#include "log.h"
#include "audio.h"
#include "clock.h"

int debug_clock;

static int unset_timer = -1;
static int set_timer = -1;
static int setting_timer = -1;
static int mushroom_timer = -1;
static int snooze_timer = -1;
static int alarm_timer = -1;
static int psychedelic_timer = -1;

static int alarm_running = 0;

static int setting_toggle;
static int setting_index;
static int64_t ref_time;

static int64_t setting_alarm;
static int alarm_min;
static int alarm_allowed = 1;

static int clock_digit[4] = {0, 0, 0, 0};
static int alarm_digit[4] = {0, 0, 0, 0};
static int *setting_digit = &clock_digit[0];

static int last_running_min;
static int ignore_next_release;

static const char *alarm_path;
static int clock_set_once;

static int psychedelic_count;

static void set_cb (void *priv);

static void unset_cb (void *priv)
{
	static int toggle = 1;
	toggle ^= 1;
	int i;
	
	//printf ("unset_cb\n");
	
	for (i = 0; i < 4; i++) {
		max_write_digit(i, toggle ? MAX_0 : MAX_NOTHING);
	}
}

static void setting_cb (void *priv)
{
	int value;
	
	log ("setting_cb\n");

	setting_toggle ^= 1;
	
	value = setting_toggle ? int2max(setting_digit[setting_index]) : MAX_NOTHING;
	if (setting_alarm && setting_index == 3)
		value |= MAX_DOT;
	max_write_digit(setting_index, value);
}

static int64_t digit2ms(int *dig)
{
	return 1000 * ((dig[0] * 10 + dig[1]) * 3600 
	             + (dig[2] * 10 + dig[3]) * 60);
}

#if 0
static void ms2digit(int64_t ms, int *dig)
{
	int64_t min = (ms / 1000) / 60;
	min %= 24 * 60;
	
	dig[3] = (min % 60) % 10;
	dig[2] = (min % 60) / 10;
	dig[1] = (min / 60) % 10;
	dig[0] = (min / 60) / 10;
}
#endif

static void min2digit(int64_t min, int *dig)
{
	min %= 24 * 60;
	
	dig[3] = (min % 60) % 10;
	dig[2] = (min % 60) / 10;
	dig[1] = (min / 60) % 10;
	dig[0] = (min / 60) / 10;
}

static void _alarm_cb (void *priv);

static void _alarm_loop(void)
{
	message_remove_handler(MESSAGE_SOUND_STOPPED, _alarm_loop);
	
	if (alarm_running && snooze_timer == -1) {
		_alarm_cb(NULL);
	}
}

static void _alarm_cb (void *priv)
{
	int running_min = ((ze_time() - ref_time) / (1000 * 60)) % (60 * 24);

	alarm_running = 1;
	alarm_timer = -1;
	
	if (alarm_path) {
		message_remove_handler(MESSAGE_SOUND_STOPPED, _alarm_loop);
		message_add_handler(MESSAGE_SOUND_STOPPED, _alarm_loop);
		audio_play_file(alarm_path);
	}
	
	if (running_min > ((alarm_min + 60) % (24 * 60))) {
		/* stop alarm afer some time */
		log("give up alarm\n");
		alarm_running = 0;
	}
}

static void snooze_cb (void *priv)
{
	snooze_timer = -1;
	
	log("snooze cb !\n");
	
	_alarm_cb(NULL);
}

static void psychedelic_cb (void*data)
{
	int val = 0;
	int i;
		
	switch (psychedelic_count%3) {
		case 0:
			val = 1;
			break;
		case 1:
			val = 0x40;
			break;
		case 2:
			val = 0x80;
			break;
	}
	
	for (i = 0; i < 4; i++) {
		max_write_digit(i, val);
	}
	
	psychedelic_count++;
}

void clock_stop_psychedelic(void)
{
	if (psychedelic_timer != -1) {
		timer_remove(&psychedelic_timer);
		timer_remove(&set_timer);
		set_timer = timer_add (set_cb, NULL, TimerRepeated, 0, 1000);
	}
}

void clock_start_psychedelic(void)
{
	if (!clock_is_set() || clock_is_alarm())
		return;
	
	timer_remove(&set_timer);

	psychedelic_count = 0;
	timer_remove(&psychedelic_timer);
	psychedelic_timer = timer_add(psychedelic_cb, NULL, TimerRepeated, 0, 50);
}


static void show_alarm(void)
{
	int i;
	
	for (i = 0; i < 4; i++) {
		int value = int2max(alarm_digit[i]);
		max_write_digit(i, value);
	}
}

static void set_cb (void *priv)
{
	int new_digit[4];
	int i;
	int running_min;
	
	static int dot;

	dot ^= MAX_DOT;

	running_min = ((ze_time() - ref_time) / (1000 * 60)) % (60 * 24);
	
	if (running_min < last_running_min) {
		/* we wrapped */
		log ("we wrapped\n");
		alarm_allowed = 1;
	}
	last_running_min = running_min;
	
	if (alarm_allowed && running_min == alarm_min) {
		printf("alarm now !!!\n");
		alarm_allowed = 0;
		if (balai_get() >= BALAI_MIDDLE) {
			printf("alarm really now\n");
			if (balai_get() == BALAI_HIGH) {
				alarm_path = "/home/root/content/raw_44100_mono_s16le/trompette_edited.raw";
			} else {
				alarm_path = "/home/root/content/raw_44100_mono_s16le/bunnies_just_wanna_have_fun.raw";
			}
			_alarm_cb(NULL);
		}
	}
	
	min2digit (running_min, new_digit);

	//printf ("set_cb old %d%d:%d%d\n", clock_digit[0], clock_digit[1], clock_digit[2], clock_digit[3]);
	//printf ("set_cb new %d%d:%d%d\n", new_digit[0], new_digit[1], new_digit[2], new_digit[3]);

	for (i = 0; i < 4; i++) {
		int value  = int2max(new_digit[i]);
		if (i == 1) {
			value |= dot;
		} else if (i == 3) {
			int alarm = 0;
			if (balai_get() >= BALAI_MIDDLE) {
				alarm = MAX_DOT;
			}
			value |= alarm;
		}

		max_write_digit(i, value);
		clock_digit[i] = new_digit[i];
	}
}

void clock_init(void)
{
	int i;

	for (i = 0; i < 4; i++) {
		max_write_digit(i, MAX_0);
	}

	unset_timer = timer_add (unset_cb, NULL, TimerRepeated, 1000, 1000);
}

void clock_start_setting(void)
{
	int i;
	
	log ("clock_start setting\n");

	timer_remove (&unset_timer);	
	timer_remove (&set_timer);

	setting_toggle = 0;
	setting_index = 0;
	
	for (i = 0; i < 4; i++) {
		if (i == 3 && setting_alarm) {
			max_write_digit(i, int2max(setting_digit[i]) | MAX_DOT);
		} else {
			max_write_digit(i, int2max(setting_digit[i]));
		}
	}
	
	setting_timer = timer_add (setting_cb, NULL, TimerRepeated, 0, 1000);
	
	ignore_next_release = 1;
}

void clock_digit_increment(void)
{
	printf ("clock_digit_increment\n");
	
	setting_digit[setting_index]++;

	if (setting_index  == 0) {
		setting_digit[setting_index] %= 3;
	} else if (setting_index == 1) {
		setting_digit[setting_index] %= 10;
	} else if (setting_index == 2) {
		setting_digit[setting_index] %= 6;
	} else if (setting_index == 3) {
		setting_digit[setting_index] %= 10;
	}

	setting_toggle = 0;

	timer_remove (&setting_timer);
	setting_timer = timer_add (setting_cb, NULL, TimerRepeated, 0, 1000);
}

void clock_set_time (int digits[4])
{
	timer_remove(&set_timer);
	timer_remove(&unset_timer);
	timer_remove (&setting_timer);

	setting_timer = -1;
	setting_index = 0;
		
	alarm_allowed = 1;

	printf("clock set %d%d:%d%d\n", 
			digits[0], digits[1],
			digits[2], digits[3]);

	ref_time = ze_time() - digit2ms(digits);  

	set_timer = timer_add (set_cb, NULL, TimerRepeated, 0, 1000);
	
	clock_set_once = 1;
}

void clock_set_alarm (int digits[4])
{
	timer_remove(&set_timer);
	timer_remove(&unset_timer);
	timer_remove (&setting_timer);

	setting_timer = -1;
	setting_index = 0;
		
	alarm_allowed = 1;

	printf("alarm set %d%d:%d%d\n", 
			digits[0], digits[1],
			digits[2], digits[3]);

	alarm_min = digit2ms(digits) / (1000 * 60);  

	set_timer = timer_add (set_cb, NULL, TimerRepeated, 0, 1000);
}

void clock_digit_change(void)
{
	max_write_digit(setting_index, int2max(setting_digit[setting_index]));

	setting_index++;
	if (setting_index == 4) {
		if (setting_alarm)
			clock_set_alarm(setting_digit);
		else
			clock_set_time(setting_digit);
	} else {
		setting_toggle = 1;

		timer_remove (&setting_timer);
		setting_timer = timer_add (setting_cb, NULL, TimerRepeated, 0, 1000);
	}
}

int clock_is_setting(void)
{
	return setting_timer != -1;
}

static void mushroom_cb (void *priv)
{
	log ("mushroom_cb\n");
	
	mushroom_timer = -1;
	
	if (!clock_is_setting()) {
		clock_start_setting();
	}
}

int clock_handle_message(int message)
{
	log("clock handle message %d\n", message);
	
	switch (message) {
		case MESSAGE_BIG_MUSHROOM_PRESSED:
			if (snooze_timer != -1 || alarm_running) {
				audio_stop();
				timer_remove(&alarm_timer);
				if (snooze_timer == -1) {
					log("snooze\n");
					snooze_timer = timer_add(snooze_cb, NULL, TimerSingle, 10 * 1000 * 60, 0);
				}
				ignore_next_release = -1;
				return 1;
			}
			
			if (!clock_is_setting()) {
				if (mushroom_timer == -1) {
					setting_alarm = 0;
					setting_digit = &clock_digit[0];
					mushroom_timer = timer_add (mushroom_cb, NULL, TimerSingle, 3000, 0);
				}
				return 0;
			}
			return 1;
		case MESSAGE_BIG_MUSHROOM_RELEASED:
			if (mushroom_timer != -1) {
				timer_remove (&mushroom_timer);
				mushroom_timer = -1;
			}

			if (ignore_next_release) {
				ignore_next_release = 0;
				return 1;
			}

			if (clock_is_setting()) {
				clock_digit_change();				
				return 1;
			} else {
				return 0;
			}
		case MESSAGE_SMALL_MUSHROOM_PRESSED:
			if (snooze_timer != -1 || alarm_running) {
				alarm_running = 0;
				timer_remove(&alarm_timer);
				timer_remove(&snooze_timer);
				audio_stop();
				ignore_next_release = 1;
				return 1;
			}

			if (clock_is_setting()) {
				return 1;
			} 
			
			if (clock_set_once) {
				if (mushroom_timer == -1) {
					setting_alarm = 1;
					setting_digit = &alarm_digit[0];
					mushroom_timer = timer_add (mushroom_cb, NULL, TimerSingle, 3000, 0);
				}
				
				timer_remove(&set_timer);
				show_alarm();
				return 1;
			}
			
			return 1;

		case MESSAGE_SMALL_MUSHROOM_RELEASED:
			if (mushroom_timer != -1) {
				timer_remove (&mushroom_timer);
			}
			if (ignore_next_release) {
				ignore_next_release = 0;
				return 1;
			}
			if (clock_is_setting()) {
				clock_digit_increment();
				return 1;
			} else {
				if (clock_set_once) {
					timer_remove(&set_timer);
					set_timer = timer_add (set_cb, NULL, TimerRepeated, 0, 1000);
				}
				return 1;
			}
		default:
			return 0;
	}
	
	return 0;
}

int clock_is_daylight(void)
{
	int64_t running_hours = ((ze_time() - ref_time) / (1000 * 60 * 60)) % 24;
	if (running_hours > 0 && running_hours <  8)
		return 0;

	return 1;
}

int clock_is_set(void)
{
	return clock_set_once;
}

int clock_is_alarm(void)
{
	return snooze_timer != -1 || alarm_running;
}


