#include <stdio.h>
#include <stdlib.h>
#include "gpio.h"
#include "timer.h"
#include "eye.h"
#include "utils.h"

#define EYE_LEFT_RED 156
#define EYE_LEFT_BLUE 666
#define EYE_RIGHT_RED 159
#define EYE_RIGHT_BLUE 131

static int eye_count;
static int eye_timer = -1;
static int rabbid;

void eye_init(void)
{
	gpio_setup(EYE_LEFT_RED, GPIO_OUTPUT, 0);
	//gpio_setup(EYE_LEFT_BLUE, GPIO_OUTPUT, 0);
	gpio_setup(EYE_RIGHT_RED, GPIO_OUTPUT, 0);
	gpio_setup(EYE_RIGHT_BLUE, GPIO_OUTPUT, 0);
}

static void _rabbid(int on)
{
	eye_left(EYE_NOTHING);

	if (on) {
		eye_right(EYE_BLUE);
	} else {
		eye_right(EYE_NOTHING);
	}
}

void eye_rabbid(int on)
{
	if (rabbid != on) {
		rabbid = on;
		_rabbid(on);
	}
}

void eye_left(int value)
{
	if (value & EYE_RED) 
		gpio_write(EYE_LEFT_RED, 1);
	else
		gpio_write(EYE_LEFT_RED, 0);

	/*if (value & EYE_BLUE) 
		gpio_write(EYE_LEFT_BLUE, 1);
	else
		gpio_write(EYE_LEFT_BLUE, 0);*/
}

void eye_right(int value)
{
	if (value & EYE_RED) 
		gpio_write(EYE_RIGHT_RED, 1);
	else
		gpio_write(EYE_RIGHT_RED, 0);

	if (value & EYE_BLUE) 
		gpio_write(EYE_RIGHT_BLUE, 1);
	else
		gpio_write(EYE_RIGHT_BLUE, 0);	
}


static void rand_cb(void *priv)
{	
	//printf("eye random %d\n", eye_count);
	
	eye_count++;

	eye_left(ze_rand()%2);
	eye_right(ze_rand()%3);
}

static void flash_cb(void *priv)
{	
	eye_count++;

	eye_left(eye_count&1);
	eye_right((eye_count&1) ? EYE_RED | EYE_BLUE : 0);
}

void eye_start_rand(void)
{
	if (eye_timer != -1) {
		timer_remove(&eye_timer);
		eye_timer = -1;
	}
	eye_count = 0;
	eye_timer = timer_add(rand_cb, NULL, TimerRepeated, 0, 50);
}

void eye_stop_rand(void)
{
	timer_remove(&eye_timer);

	_rabbid(rabbid);
}

void eye_start_flash(void)
{
	if (eye_timer != -1) {
		timer_remove(&eye_timer);
		eye_timer = -1;
	}
	eye_count = 0;
	eye_timer = timer_add(flash_cb, NULL, TimerRepeated, 0, 50);
}

void eye_stop_flash(void)
{
	timer_remove(&eye_timer);

	_rabbid(rabbid);
}

