#include <stdio.h>
#include <unistd.h>
#include <timer.h>
#include <stdlib.h>
#include <inttypes.h>

#include "gpio.h"
#include "max.h"
#include "clock.h"
#include "timer.h"
#include "message.h"
#include "mushroom.h"
#include "eye.h"
#include "balai.h"
#include "audio.h"
#include "log.h"
#include "quotes.h"
#include "utils.h"
#include "sounds.h"

int debug = 0;

extern void monitor_init(void);

#define STUPID_PERIOD	30

static int stupid_id = -1;
static int sound_id;
static int rabbid = 0;
static int intensity_id = -1;

static int next_sound_id;
static int _stupid_delay(void)
{
	int min;
	
	min = STUPID_PERIOD / 2 + (ze_rand() % STUPID_PERIOD);
	
	return min * 60 * 1000;
}

static void _stupid_cb(void *data)
{
	
	stupid_id = -1;
	stupid_id = timer_add(_stupid_cb, NULL, TimerSingle, _stupid_delay(), 0);

	if (!clock_is_set() || clock_is_alarm()) {
		log_err("no stupidities if clock not set or alarm\n");
		return ;
	}

	if (balai_get() == BALAI_HIGH && clock_is_daylight()) {
		switch (ze_rand() % 1) {
			case 0:
				quotes_random();
				break;
			default:
				break;
		}
	}
}

static void _intensity_cb(void *data)
{
	intensity_id = -1;
	
	max_intensity(1);
}

static void _stop_everything (void)
{
	eye_stop_flash();
	eye_stop_rand();
	clock_stop_psychedelic();

	message_remove_handler(MESSAGE_SOUND_STOPPED, _stop_everything);	
}

static void _bwaaah(int message)
{
	static int64_t last_time;
	static int count;
	
	if (message != MESSAGE_BIG_MUSHROOM_RELEASED)
		return;

	if (ze_time() - last_time < 1500) {
		count++;
	} else {
		count = 0;

		if (next_sound_id != -1) {
			sound_id = next_sound_id;
			next_sound_id = -1;
		} else {
			sound_id = ze_rand() % sound_short_count();
		}
	}

	last_time = ze_time();
	
	if (clock_is_daylight() || balai_get() == BALAI_HIGH) {

		if (count < 10) {
			eye_start_rand();
			clock_start_psychedelic();
			sound_play_short(sound_id);
		} else if (count == 10) {
			eye_start_flash();
			clock_start_psychedelic();
			next_sound_id = sound_id;
			switch (ze_rand() % 3) {
				case 0:
					audio_play_string("arrête de m'emmerder !");
					break;
				default:
					sound_play_rand();
					break;
			}
		}
		
		message_remove_handler(MESSAGE_SOUND_STOPPED, _stop_everything);
		message_add_handler(MESSAGE_SOUND_STOPPED, _stop_everything);
	}
}
int main(int argc, char *argv[]) 
{
	printf("Bwwaaaaaa\n");

	gpio_init();
	
	max_init();
	balai_init();
	audio_init();	
	timer_init();	
	mushroom_init();
	clock_init();
	eye_init();
	quotes_init();
	
	monitor_init();
	
	stupid_id = timer_add(_stupid_cb, NULL, TimerSingle, _stupid_delay(), 0);
	intensity_id = timer_add(_intensity_cb, NULL, TimerSingle, 10000, 0);
				
	rabbid = (balai_get() == BALAI_HIGH);
	
	while (1) {
		int message;
		int64_t max_sleep = timer_next_timeout() - ze_time();
		
		/* psychological */
		if (max_sleep < 0)
			max_sleep = 0;
		max_sleep += 1;
			
		if (!message_num()) {
			message_wait(max_sleep);
		}
				
		timer_trigger();		

		if (balai_get() == BALAI_HIGH) {
			if (!rabbid) {
				eye_rabbid(1);
				quotes_random();
				rabbid = 1;
			}
		} else {
			eye_rabbid(0);
			rabbid = 0;
		}
		
		while (message_get(&message)) {
			if (intensity_id == -1) {
				max_intensity(0xf);
			}
			if (intensity_id != -1)
				timer_remove(&intensity_id);
			intensity_id = timer_add(_intensity_cb, NULL, TimerSingle, 10000, 0);
			
			if (!clock_handle_message(message)) {				
				_bwaaah(message);
			}
			message_handle (message);
		}
	}
	
	return 0;
}

