#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <errno.h>
#include "gpio.h"
#include "max.h"

#define LOG_CONDITION debug_max

#include "log.h"

int debug_max;

static const int load = 139;
static const int clock = 138;
static const int din = 137;

//#define DELAY do {} while (0);
#define DELAY usleep(1);
#define HIGH 0
#define LOW 1

//#define USE_MODULE

int int2max(int value)
{
	switch (value) {
		case 0: return MAX_0;
		case 1: return MAX_1;
		case 2: return MAX_2;
		case 3: return MAX_3;
		case 4: return MAX_4;
		case 5: return MAX_5;
		case 6: return MAX_6;
		case 7: return MAX_7;
		case 8: return MAX_8;
		case 9: return MAX_9;
		default: return MAX_NOTHING;
	}
}

void max_intensity(int value)
{
	log("intensity %d\n", value);
	max_write_register(MAX_REGISTER_INTENSITY, value);
}

#ifndef USE_MODULE

int max_powerup(void)
{
	gpio_setup(load, GPIO_OUTPUT, 0);
	gpio_setup(clock, GPIO_OUTPUT, 0);
	gpio_setup(din, GPIO_OUTPUT, 0);
	
	return 0;
}

int max_init(void)
{
	int i;
	log("max_init\n");
	gpio_setup(load, GPIO_OUTPUT, 0);
	gpio_setup(clock, GPIO_OUTPUT, 0);
	gpio_setup(din, GPIO_OUTPUT, 0);

	log("disable shutdown\n");
	max_write_register(MAX_REGISTER_SHUTDOWN, 1);
	log("zero all digits\n");
	for (i = 1; i <= 4; i++) {
		max_write_register(i, 0);
	}
	log("scan all digits\n");
	max_write_register(MAX_REGISTER_SCAN_LIMIT, 3);
	log("disable test\n");
	max_write_register(MAX_REGISTER_TEST, 0);
	log("disable decode\n");
	max_write_register(MAX_REGISTER_DECODE, 0);
	log("intensity max\n");
	max_write_register(MAX_REGISTER_INTENSITY, 0xf);

	return 0;
}

void max_write_register(int addr, int value)
{
	int bits = addr << 8 | value;
	int i;

	log("write %02x in %02x (bits = %04x)\n", value, addr, bits);

	gpio_write(clock, LOW);
	gpio_write(load, LOW);
	DELAY;
	
	for (i = 15; i >= 0; i--) {
		gpio_write(din, (bits & (1 << i)) ? HIGH : LOW);
		DELAY;
		gpio_write(clock, HIGH);
		DELAY;
		gpio_write(clock, LOW);
		DELAY;
	}

	gpio_write(load, HIGH);
	DELAY;
	gpio_write(load, LOW);
	DELAY;
}

void max_write_digit(int index, int value)
{
	int addr;
	
	switch (index) {
		case 0: addr = 2; break;
		case 1: addr = 4; break;
		case 2: addr = 3; break;
		case 3: addr = 1; break;
		default: return;		
	}
	
	max_write_register (addr, value);
}

#else
#include "modules/max7219/max7219.h"
static int max7219_fd = -1;

int max_init(void)
{
	gpio_setup(load, GPIO_OUTPUT, 0);
	gpio_setup(clock, GPIO_OUTPUT, 0);
	gpio_setup(din, GPIO_OUTPUT, 0);

	system("insmod /home/root/max7219.ko");

	max7219_fd = open("/dev/max7219", O_RDONLY);
	if (max7219_fd == -1) {
		printf("cannot open max7219 device\n");	
	}
	
	return 0;
}

void max_write_register(int addr, int value)
{
}

void max_write_digit(int index, int value)
{
	Max7219Command command;
	
	command.index = index;
	command.value = value;
	
	if (ioctl(max7219_fd, MAX7219_COMMAND, &command) < 0) {
		printf("cannot ioctl (%s)\n", strerror(errno));	
		return;	
	}
}

#endif

