#include <pthread.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include "message.h"
#include "log.h"

#define MESSAGE_MAX 16

int queue[MESSAGE_MAX];
int queue_read;
int queue_write;
int queue_num;

static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t cond = PTHREAD_COND_INITIALIZER;

#define MAX_HANDLERS 10

static struct {
	int message;
	void (*handler)(void);
} _handler[MAX_HANDLERS];

void message_put(int message)
{
	pthread_mutex_lock(&mutex);
	
	//printf("put %d\n", message);
	if (queue_num < MESSAGE_MAX) {
		queue[queue_write] = message;
		queue_write = (queue_write + 1) % MESSAGE_MAX;
		queue_num++;
		
		pthread_cond_signal(&cond);
	} else {
		printf("queue full\n");
	}

	pthread_mutex_unlock(&mutex);
}

int message_get(int *message)
{
	pthread_mutex_lock(&mutex);
		
	if (queue_num == 0) {
		pthread_mutex_unlock(&mutex);
		return 0;
	} else {
		int m = queue[queue_read];

		queue_read = (queue_read + 1) % MESSAGE_MAX;
		
		*message = m;

		queue_num--;
		
		pthread_mutex_unlock(&mutex);
		
		return 1;
	}
}

void message_wait(int timeout)
{
	struct timeval now;
	struct timespec tv;
	unsigned long nano;

	gettimeofday(&now, NULL);
	nano = (now.tv_usec + (timeout % 1000) * 1000) * 1000;
	tv.tv_nsec = nano % (1000 * 1000 * 1000);
	tv.tv_sec = now.tv_sec + timeout/1000 + nano / (1000 * 1000 * 1000);

	pthread_mutex_lock(&mutex);
	pthread_cond_timedwait(&cond, &mutex, &tv);
	pthread_mutex_unlock(&mutex);
}

int message_num(void)
{
	return queue_num;
}

void message_add_handler(int message, void (*handler)(void))
{
	int i;
	
	for (i = 0; i < MAX_HANDLERS; i++) {
		if (_handler[i].handler == NULL) {
			_handler[i].message = message;
			_handler[i].handler = handler;
			
			return;
		}
	}
	
	log_err ("too many handlers\n");
}

void message_remove_handler(int message, void (*handler)(void))
{
	int i;
	
	for (i = 0; i < MAX_HANDLERS; i++) {
		if (_handler[i].handler == handler && _handler[i].message == message) {
			_handler[i].handler = NULL;
		}
	}
}

void message_handle(int message)
{
	int i;
	
	for (i = 0; i < MAX_HANDLERS; i++) {
		if (_handler[i].message == message) {
			if (_handler[i].handler != NULL) {
				_handler[i].handler();
			}
		}
	}
}

