#include <unistd.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>

#include "log.h"
#include "balai.h"
#include "clock.h"
#include "audio.h"
#include "quotes.h"
#define MONITOR_FD 0

static pthread_t thread;

extern int debug_audio;
extern int debug_mushroom;
extern int debug_gpio;
extern int debug_clock;
extern int debug_max;
extern int debug_espeak;

static void monitor_do(const char *buf)
{
	const char *param = strchr(buf, ' ');
	if (param) {
		param++;
	}
	
	if (buf[0] == 'd') {
		int *d = NULL;
		switch (buf[1]) {
			case 'a':
				d = &debug_audio;
				break;
			case 'g':
				d = &debug_gpio;
				break;
			case 'm':
				if (buf[2] == 'u')
					d = &debug_mushroom;
				else
					d = &debug_max;
				break;
			case 'c':
				d = &debug_clock;
				break;
			case 'e':
				d = &debug_espeak;
				break;
			default:
				d = &debug;
				break;			
		}
		if (d) {
			*d ^= 1;
			log_err ("debug %s is now %s\n", buf, *d ? "ON" : "OFF");
		}
	} else if (!strncmp(buf, "balai", 5)) {
		if (param) {
			balai_force = atoi(param);
			log_err ("balai_force = %d\n", balai_force);
		} else {
			log_err ("balai [-1|0|1|2] (auto|low|middle|high)\n");
		}
	} else if (!strncmp(buf, "time", 4) || !strncmp(buf, "alarm", 5)) {
		if (param && strlen(param) == 4) {
			int digits[4];
			int i;
			for (i = 0; i < 4; i++) {
				digits[i] = param[i] - '0';
			}
			
			if (!strncmp(buf, "time", 4)) {
				clock_set_time(digits);
			} else {
				clock_set_alarm(digits);
			}
		} else {
			log_err ("clock|alarm [YYXX]\n");
		}
	} else if (!strncmp(buf, "speak", 5)) {
		if (param)
			audio_play_string(param);
	} else if (!strncmp(buf, "quote", 5)) {
		quotes_random();
	} else if (buf[0] == 'q'){
		log_err ("Bwoooooooooooooo\n");
		exit(1);
	} else if (buf[0] != '\0'){
		log_err ("ze lapin just knows about:\n");
		log_err ("\tdebug\n");
		log_err ("\tbalai\n");
		log_err ("\ttime\n");
		log_err ("\talarm\n");
		log_err ("\tspeak\n");
		log_err ("\tquote\n");
	}
}

static void * monitor_thread(void *toto)
{
	char buf[256];
	char *p = buf;
	
	write(1, ">", 1);
	while (1) {
		if (read(MONITOR_FD, p, 1) == 1) {
			if (*p == '\n') {
				*p = '\0';
				monitor_do(buf);
				p = buf;
				write(1, ">", 1);
				continue;
			}
			
			p++;
		} else {
			p = buf;
			continue;
		}		
	}
}

void monitor_init (void)
{
	thread = pthread_create(&thread, NULL, monitor_thread, NULL);
}
