#include <stdint.h>
#include <stdio.h>
#include <pthread.h>
#include <unistd.h>
#include "message.h"
#include "sense.h"
#include "timer.h"

#define LOG_CONDITION debug_mushroom
#include "log.h"

int debug_mushroom;

static pthread_t thread;

static uint64_t calibration_a;
static uint64_t calibration_b;

static int a_pressed;
static int b_pressed;

#define AVG_SIZE 200

static int avg_a[AVG_SIZE];
static int avg_b[AVG_SIZE];

int64_t acc_a;
int64_t acc_b;

int avg_index;

static void _push(int a, int b)
{
	int new_index = (avg_index + 1) % AVG_SIZE;
	
	int old_a = avg_a[new_index];
	int old_b = avg_b[new_index];

	avg_a[new_index] = a;
	avg_b[new_index] = b;
	
	acc_a += (a - old_a);
	acc_b += (b - old_b);
	
	avg_index = new_index;
}

static void _read(uint64_t *total_a, uint64_t *total_b, int retries)
{
	int i;
	int a, b;

	for (i = 0; i < retries; i++) {
		sense_detect(&a, &b);
		_push(a, b);
		
		*total_a += a;
		*total_b += b;
	}

	*total_a /= (retries * 1000);
	*total_b /= (retries * 1000);
}

void *mushroom_thread(void*priv)
{
	int a, b;
	int aa, bb;
	int last_time = ze_time();
			
	while (1) {
		sense_detect(&a, &b);
		if (a / 1000 > 3 * calibration_a || b / 1000 > 3 * calibration_b) {
			//log("drop spurious value %10d - %10d\n", a / 1000, b / 1000);
		} else {
			_push(a, b);
		}
	
		aa = acc_a / (1000 * AVG_SIZE);
		bb = acc_b / (1000 * AVG_SIZE);

		if (ze_time() - last_time > 200) {
			log("mushroom: %4d - %4d\n", aa, bb);
			last_time = ze_time();
		}
	
		if (100 * aa >  123 * calibration_a) {
			if (!a_pressed) {
				a_pressed = 1;
				message_put(MESSAGE_BIG_MUSHROOM_PRESSED);
			}
		} else if (100 * aa <  105 * calibration_a){
			if (a_pressed) {
				a_pressed = 0;
				message_put(MESSAGE_BIG_MUSHROOM_RELEASED);
			}
		}
		if (100 * bb >  123 * calibration_b) {
			if (!b_pressed) {
				b_pressed = 1;
				message_put(MESSAGE_SMALL_MUSHROOM_PRESSED);
			}
		} else if (100 * bb <  105 * calibration_b){
			if (b_pressed) {
				b_pressed = 0;
				message_put(MESSAGE_SMALL_MUSHROOM_RELEASED);
			}
		}
	}
}

void mushroom_init(void)
{
	struct sched_param sched_param;
	int64_t start;
	
	sense_init();

	pthread_attr_t thread_attr;
	pthread_attr_init(&thread_attr);
	pthread_attr_setschedpolicy(&thread_attr, SCHED_FIFO);
	pthread_attr_getschedparam(&thread_attr, &sched_param);
	sched_param.sched_priority = 10;
	pthread_attr_setschedparam(&thread_attr, &sched_param);

	start = ze_time();
	
	_read(&calibration_a, &calibration_b, 1000);
	
	log("mushroom: calibration %10lld - %10lld (took %lldms)\n", calibration_a, calibration_b, ze_time() - start);

	thread = pthread_create(&thread, NULL /*&thread_attr*/, mushroom_thread, NULL);
}
