#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <arpa/inet.h>

#include "log.h"
#include "audio.h"
#include "utils.h"

typedef struct {
	uint32_t offset;
	uint32_t length;
} QuoteEntry;

static QuoteEntry *entry;
static int entry_count;
static int quotes_fd = -1;

void quotes_init(void)
{
	struct stat st;
	int i;
	int fd = open("/home/root/wikiquote/index", O_RDONLY);
	if (fd == -1) {
		log_err ("cannot open quotes\n");
		return;
	}
	
	if (fstat(fd, &st) == -1) {
		log_err ("cannot stat\n");
		return;
	}
	
	entry_count = st.st_size / 8;

	entry = malloc(entry_count * sizeof(QuoteEntry));
	
	if (read(fd, entry, st.st_size) != st.st_size) {
		log_err ("cannot read index: %s\n", strerror(errno));
		return;
	}
	
	for (i = 0; i < entry_count; i++) {
		entry[i].offset = ntohl(entry[i].offset);
		entry[i].length = ntohl(entry[i].length);
		//printf ("%08x - %08x\n", entry[i].offset, entry[i].length);
	}
	
	close(fd);
	
	quotes_fd = open("/home/root/wikiquote/quotes_sanitized", O_RDONLY);

	if (quotes_fd == -1) {
		log_err ("cannot open quotes_sanitized\n");
	}
}

void quotes_random(void)
{
	int i;
	char buf[512] = {0};
	QuoteEntry *e;
	
	if (!entry_count) {
		log_err("no quotes\n");
		return;
	}
	
	i = ze_rand() % entry_count;
	e = &entry[i];
	lseek(quotes_fd, e->offset, SEEK_SET);
	
	if (e->length > sizeof(buf) - 1) {
		log_err ("quote is too long\n");
		return;
	}
	if (read (quotes_fd, buf, e->length) != e->length) {
		log_err ("cannot read quote\n");
		return;
	}
	
	audio_play_string(buf);
}

