#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include "gpio.h"
#include "sense.h"
#include "log.h"

#define PIN_SEND_A 130
#define PIN_RECEIVE_A 183
#define PIN_RECEIVE_B 168

#define USE_MODULE

#ifdef USE_MODULE
#include "modules/capsense/capsense.h"
int capsense_fd = -1;
#endif

#define CLOCK_ID CLOCK_MONOTONIC

void sense_init(void)
{
	gpio_setup(PIN_SEND_A, GPIO_OUTPUT, 0);
	gpio_setup(PIN_RECEIVE_A, GPIO_INPUT, GPIO_NONE);
	gpio_setup(PIN_RECEIVE_B, GPIO_INPUT, GPIO_NONE);
#ifdef USE_MODULE
	system("insmod /home/root/capsense.ko");
	capsense_fd = open("/dev/capsense", O_RDONLY);
	if (capsense_fd == -1) {
		printf("cannot open capsense device\n");	
	}
#endif
}

#ifndef USE_MODULE
int sense_detect(int *a, int *b) 
{
	uint64_t val;
	struct timespec start;
	struct timespec enda;
	struct timespec endb;
	struct timespec res;
	int got_a = 0;
	int got_b = 0;

	clock_getres(CLOCK_ID, &res);

	gpio_write(PIN_SEND_A, 0);
	usleep(200);

	clock_gettime(CLOCK_ID, &start);
	gpio_write(PIN_SEND_A, 1);

	while(!got_a || !got_b) {
		if (!got_a && gpio_read(PIN_RECEIVE_A)) {
			clock_gettime(CLOCK_ID, &enda);
			got_a = 1;
		}
		if (!got_b && gpio_read(PIN_RECEIVE_B)) {
			clock_gettime(CLOCK_ID, &endb);
			got_b = 1;
		}
	}

	gpio_write(PIN_SEND_A, 0);

	val = (enda.tv_sec - start.tv_sec) * 1000000000;
	val += enda.tv_nsec - start.tv_nsec;

	*a = val;

	val = (endb.tv_sec - start.tv_sec) * 1000000000;
	val += endb.tv_nsec - start.tv_nsec;

	*b = val;
	
	return 0;
}

#else
int sense_detect(int *a, int *b) 
{
	int value[2];
	
	if (ioctl(capsense_fd, CAPSENSE_READ, &value[0]) < 0) {
		printf("cannot sense ioctl: %s\n", strerror(errno));	
		return -1;	
	}
	
	*a = value[0];
	*b = value[1];
	return 0;
}
#endif
